﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------
	
-------------------------------------------------------------------------------
-- default layout values
--
-- notes:
--
-- casting bar layouts are performed using the full inside height of the status bars box. If
-- a latency bar will be shown, its size is fixed and it is located at the bottom of the status
-- bar box. The height of the latency bar is deducted from the available vertical lines before
-- any other calculations are done. By default, the remaining vertical lines are divided
-- equally by the number of bars to be displayed within the status bar box. However, if either
-- the casting bar or power bar options are set to "thin" then their height is divided by two
-- and the balance is added to the height of the health bar

nUI_UnitLayout = 
{
	unit_id       = "",			-- player, target, targettarget, etc.
	name          = nil,		-- root name for the unit: default is "nUI_Unit"
	id            = nil,		-- numerical ID used in groups/parties: nil or 1..n
	is_visible    = "yes",		-- whether or not to display the unit
	class_colors  = "yes",		-- whether or not to colorize name labels by class: yes|no
	show_reaction = "yes",		-- whether or not to highlight label bar with mob reaction: yes|no
	orientation   = "left",		-- how to justify the unit's content and portrait: left|right
	show_detail   = "yes",		-- show the unit level and class: yes|no
	show_border   = "no",		-- show a background border even when unit does not exist: yes|no
	show_class    = "yes",		-- whether or not to show the unit's class along with level: yes|no
	elite_icon    = "no",		-- show an icon to indicate the unit is elite: yes|no

	Size =
	{
		width     = 515,		-- self explanatory... right?
		height    = 150,
		two_line  = "no",		-- use 3 line (label/level/class) or 2 line (label/level+class): yes|no
		compact   = "no",		-- place health/power bars inside the label and make one line: yes/no
	},
	Origin =
	{
		anchor    = "CENTER",	-- the anchor point we measure to (TOPLEFT, CENTER, BOTTOM, etc.)
		xOfs      = 0,			-- horizontal offset from the center of the parent to the anchor
		yOfs      = 0,			-- vertical offset from the center of the parent to the anchor
	},
	Portrait =
	{
		visible   = "anim",		-- whether or not to include a portrait frame: yes|no|anim
		style     = "square",	-- portrait style to use if shown: square|circle
	},
	
	StatusBars =
	{
		visible   = "yes",		-- whether or not to include health/power bars: yes|no
		power     = "normal",	-- what type of power bar to display: normal|thin|none
		direction = "right",	-- which way does the bar grow: left|right
		animate   = "no",		-- whether or not to animate the bar: yes|no
		spell     = "center",   -- whether or not to show the cast bar label: min|max|center|none
		name      = "none",		-- whether or not to show bar name: min|max|center|outside|none
		percent   = "none",		-- whether or not to show percent of total: min|max|center|outside|none
		values    = "center",	-- whether or not to show current value: min|max|center|outside|none
		total     = "center",	-- whether or not to show max value: min|max|center|outside|none
	},	

	CastingBars =
	{
		visible   = "no",		-- whether or not to include a casting bar: yes|no
		latency   = "no",		-- whether or not to include a latency bar: yes|no
		name      = "min",		-- whether or not to show bar name: min|max|center|outside|none
		percent   = "max",		-- whether or not to show percent of total: min|max|center|outside|none
		values    = "none",	    -- whether or not to show current value: min|max|center|outside|none
		total     = "none",		-- whether or not to show max value: min|max|center|outside|none
	},
	
	BuffInfo =
	{
		Buffs =
		{
			location   = "hidden",	-- where to display unit buffs: top|bottom|left|right|detatched|hidden|none
			anchor     = "",		-- if detatched, what anchor point is used on the buff bar
			flash      = 0,			-- when to start flashing expiring buffs (0 = never)
			flash_rate = 1.5,		-- how fast to flash them (in seconds)
			timers     = "no",		-- whether or not to display time remaining counters: yes|no
			anim       = "no",		-- whether or not to display the Bliz cooldown animation: yes|no
			highlight  = "no",		-- whether or not to highlight icons by type: yes|no
			xOfs       = 0,			-- horizontal offset when detatched
			yOfs       = 0,			-- vertical offset when detatched
			width      = nil,		-- overrides the sizing of the frame matching the unit width
			type       = "buff",	-- buff type to diplay: buff|debuff|adapt|adapt-|all|none
			rows       = 5,			-- how many rows (colums for left and right locations) to display
			gap        = 2,			-- the space between adjacent icons
		},
		Debuffs =
		{
			location   = "hidden",	-- where to display unit debuffs: top|bottom|left|right|detatched|hidden|none
			anchor     = "",		-- if detacched, what anchor point is used on the buff bar
			flash      = 0,			-- when to start flashing expiring buffs (0 = never)
			flash_rate = 1.5,		-- how fast to flash them (in seconds)
			timers     = "no",		-- whether or not to display time remaining counters: yes|no
			anim       = "no",		-- whether or not to display the Bliz cooldown animation: yes|no
			highlight  = "no",		-- whether or not to highlight icons by type: yes|no
			xOfs       = 0,			-- horizontal offset when detatched
			yOfs       = 0,			-- vertical offset when detatched
			width      = nil,		-- overrides the sizing of the frame matching the unit width
			type       = "debuff",	-- buff type to diplay: buff|debuff|adapt|adapt-|all|none
			rows       = 5,			-- how many rows (colums for left and right locations) to display
			gap        = 2,			-- the space between adjacent icons
		},
	},
}

nUI.Class.UnitLayout = nUI.Class:CreatePrototype();
local prototype      = nUI.Class.UnitLayout.prototype;

-------------------------------------------------------------------------------
-- this may look a little like a PITA, but it makes it possible to add new values
-- at any time without having to parse the existing user config and insert, etc.

function prototype:init( unit_id, user_defined )

	if not user_defined then user_defined = {}; end
	if not user_defined.Size then user_defined.Size = {}; end
	if not user_defined.Origin then user_defined.Origin = {}; end
	if not user_defined.Portrait then user_defined.Portrait = {}; end
	if not user_defined.StatusBars then user_defined.StatusBars = {}; end
	if not user_defined.CastingBars then user_defined.CastingBars = {}; end
	if not user_defined.BuffInfo then user_defined.BuffInfo = {}; end
	if not user_defined.BuffInfo.Buffs then user_defined.BuffInfo.Buffs = {}; end
	if not user_defined.BuffInfo.Debuffs then user_defined.BuffInfo.Debuffs = {}; end
	if not user_defined.HealthBar then user_defined.HealthBar = {}; end
	if not user_defined.ManaBar then user_defined.ManaBar = {}; end
	if not user_defined.RageBar then user_defined.RageBar = {}; end
	if not user_defined.EnergyBar then user_defined.EnergyBar = {}; end
	if not user_defined.FocusBar then user_defined.FocusBar = {}; end
	if not user_defined.LatencyBar then user_defined.LatencyBar = {}; end
	if not user_defined.CastBar then user_defined.CastBar = {}; end
	if not user_defined.ChannelBar then user_defined.ChannelBar = {}; end
	if not user_defined.HealthBar.Options then user_defined.HealthBar.Options = {}; end
	if not user_defined.ManaBar.Options then user_defined.ManaBar.Options = {}; end
	if not user_defined.RageBar.Options then user_defined.RageBar.Options = {}; end
	if not user_defined.EnergyBar.Options then user_defined.EnergyBar.Options = {}; end
	if not user_defined.FocusBar.Options then user_defined.FocusBar.Options = {}; end
	if not user_defined.LatencyBar.Options then user_defined.LatencyBar.Options = {}; end
	if not user_defined.CastBar.Options then user_defined.CastBar.Options = {}; end
	if not user_defined.ChannelBar.Options then user_defined.ChannelBar.Options = {}; end

	self.unit_id       = strlower( unit_id );
	self.name          = user_defined.name or nUI_UnitLayout.name;
	self.id            = user_defined.id or nUI_UnitLayout.id;
	self.is_visible    = strlower( user_defined.is_visible or nUI_UnitLayout.is_visible );
	self.class_colors  = strlower( user_defined.class_colors or nUI_UnitLayout.class_colors );
	self.show_reaction = strlower( user_defined.show_reaction or nUI_UnitLayout.show_reaction );
	self.orientation   = strlower( user_defined.orientation or nUI_UnitLayout.orientation );
	self.show_detail   = strlower( user_defined.show_detail or nUI_UnitLayout.show_detail );
	self.show_border   = strlower( user_defined.show_border or nUI_UnitLayout.show_border );
	self.show_class    = strlower( user_defined.show_class or nUI_UnitLayout.show_class );
	self.elite_icon    = strlower( user_defined.elite_icon or nUI_UnitLayout.elite_icon );

	self.Size = 
	{
		width          = tonumber( user_defined.Size.width or nUI_UnitLayout.Size.width ),
		height         = tonumber( user_defined.Size.height or nUI_UnitLayout.Size.height ),
		two_line       = strlower( user_defined.Size.two_line or nUI_UnitLayout.Size.two_line ),
		compact        = strlower( user_defined.Size.compact or nUI_UnitLayout.Size.compact ),
	};
	
	self.Origin = 
	{
		anchor         = strupper( user_defined.Origin.anchor or nUI_UnitLayout.Origin.anchor ),
		xOfs           = tonumber( user_defined.Origin.xOfs or nUI_UnitLayout.Origin.xOfs ),
		yOfs           = tonumber( user_defined.Origin.yOfs or nUI_UnitLayout.Origin.yOfs ),
	};
	
	self.Portrait = 
	{
		visible        = strlower( user_defined.Portrait.visible or nUI_UnitLayout.Portrait.visible ),
		style          = strlower( user_defined.Portrait.style or nUI_UnitLayout.Portrait.style ),
	};
	
	self.StatusBars = 
	{
		visible        = strlower( user_defined.StatusBars.visible or nUI_UnitLayout.StatusBars.visible ),
		power          = strlower( user_defined.StatusBars.power or nUI_UnitLayout.StatusBars.power ),
		direction      = strlower( user_defined.StatusBars.direction or nUI_UnitLayout.StatusBars.direction ),
		animate        = strlower( user_defined.StatusBars.animate or nUI_UnitLayout.StatusBars.animate ),
		name           = strlower( user_defined.StatusBars.name or nUI_UnitLayout.StatusBars.name ),
		percent        = strlower( user_defined.StatusBars.percent or nUI_UnitLayout.StatusBars.percent ),
		values         = strlower( user_defined.StatusBars.values or nUI_UnitLayout.StatusBars.values ),
		total          = strlower( user_defined.StatusBars.total or nUI_UnitLayout.StatusBars.total ),
	};

	self.CastingBars =
	{
		visible        = strlower( user_defined.CastingBars.visible or nUI_UnitLayout.CastingBars.visible ),
		latency        = strlower( user_defined.CastingBars.latency or nUI_UnitLayout.CastingBars.latency ),
		name           = strlower( user_defined.CastingBars.name or nUI_UnitLayout.CastingBars.name ),
		percent        = strlower( user_defined.CastingBars.percent or nUI_UnitLayout.CastingBars.percent ),
		values         = strlower( user_defined.CastingBars.values or nUI_UnitLayout.CastingBars.values ),
		total          = strlower( user_defined.CastingBars.total or nUI_UnitLayout.CastingBars.total ),
	};
	
	self.BuffInfo = 
	{
		Buffs =
		{
			location   = strlower( user_defined.BuffInfo.Buffs.location or nUI_UnitLayout.BuffInfo.Buffs.location ),
			anchor     = strupper( user_defined.BuffInfo.Buffs.anchor or nUI_UnitLayout.BuffInfo.Buffs.anchor ),
			flash      = tonumber( user_defined.BuffInfo.Buffs.flash or nUI_UnitLayout.BuffInfo.Buffs.flash ),
			flash_rate = tonumber( user_defined.BuffInfo.Buffs.flash_rate or nUI_UnitLayout.BuffInfo.Buffs.flash_rate ),
			timers     = strlower( user_defined.BuffInfo.Buffs.timers or nUI_UnitLayout.BuffInfo.Buffs.timers ),
			anim       = strlower( user_defined.BuffInfo.Buffs.anim or nUI_UnitLayout.BuffInfo.Buffs.anim ),
			highlight  = strlower( user_defined.BuffInfo.Buffs.highlight or nUI_UnitLayout.BuffInfo.Buffs.highlight ),
			xOfs       = tonumber( user_defined.BuffInfo.Buffs.xOfs or nUI_UnitLayout.BuffInfo.Buffs.xOfs ),
			yOfs       = tonumber( user_defined.BuffInfo.Buffs.yOfs or nUI_UnitLayout.BuffInfo.Buffs.yOfs ),
			width      = user_defined.BuffInfo.Buffs.width or nUI_UnitLayout.BuffInfo.Buffs.width,
			type       = strlower( user_defined.BuffInfo.Buffs.type or nUI_UnitLayout.BuffInfo.Buffs.type ),
			rows       = tonumber( user_defined.BuffInfo.Buffs.rows or nUI_UnitLayout.BuffInfo.Buffs.rows ),
			gap        = tonumber( user_defined.BuffInfo.Buffs.gap or nUI_UnitLayout.BuffInfo.Buffs.gap ),
		},
		Debuffs =
		{
			location   = strlower( user_defined.BuffInfo.Debuffs.location or nUI_UnitLayout.BuffInfo.Debuffs.location ),
			anchor     = strupper( user_defined.BuffInfo.Debuffs.anchor or nUI_UnitLayout.BuffInfo.Debuffs.anchor ),
			flash      = tonumber( user_defined.BuffInfo.Debuffs.flash or nUI_UnitLayout.BuffInfo.Debuffs.flash ),
			flash_rate = tonumber( user_defined.BuffInfo.Debuffs.flash_rate or nUI_UnitLayout.BuffInfo.Debuffs.flash_rate ),
			timers     = strlower( user_defined.BuffInfo.Debuffs.timers or nUI_UnitLayout.BuffInfo.Debuffs.timers ),
			anim       = strlower( user_defined.BuffInfo.Debuffs.anim or nUI_UnitLayout.BuffInfo.Debuffs.anim ),
			highlight  = strlower( user_defined.BuffInfo.Debuffs.highlight or nUI_UnitLayout.BuffInfo.Debuffs.highlight ),
			xOfs       = tonumber( user_defined.BuffInfo.Debuffs.xOfs or nUI_UnitLayout.BuffInfo.Debuffs.xOfs ),
			yOfs       = tonumber( user_defined.BuffInfo.Debuffs.yOfs or nUI_UnitLayout.BuffInfo.Debuffs.yOfs ),
			width      = user_defined.BuffInfo.Debuffs.width or nUI_UnitLayout.BuffInfo.Debuffs.width,
			type       = strlower( user_defined.BuffInfo.Debuffs.type or nUI_UnitLayout.BuffInfo.Debuffs.type ),
			rows       = tonumber( user_defined.BuffInfo.Debuffs.rows or nUI_UnitLayout.BuffInfo.Debuffs.rows ),
			gap        = tonumber( user_defined.BuffInfo.Debuffs.gap or nUI_UnitLayout.BuffInfo.Debuffs.gap ),
		},
	};

	self.HealthBar  = nUI.Class.UnitBar_HealthLayout:new( user_defined.HealthBar );
	self.ManaBar    = nUI.Class.UnitBar_ManaLayout:new(  user_defined.ManaBar );
	self.RageBar    = nUI.Class.UnitBar_RageLayout:new( user_defined.RageBar );
	self.EnergyBar  = nUI.Class.UnitBar_EnergyLayout:new( user_defined.EnergyBar );
	self.FocusBar   = nUI.Class.UnitBar_FocusLayout:new( user_defined.FocusBar );
	self.LatencyBar = nUI.Class.UnitBar_LatencyLayout:new( user_defined.LatencyBar );
	self.CastBar    = nUI.Class.UnitBar_CastingLayout:new( user_defined.CastBar );
	self.ChannelBar = nUI.Class.UnitBar_ChannelingLayout:new( user_defined.ChannelBar );	

	self.HealthBar.name               = "nUI_Unit_"..self.unit_id.."_HealthBar";
	self.HealthBar.Options.show_name  = strlower( user_defined.StatusBars.name or nUI_UnitLayout.StatusBars.name );
	self.HealthBar.Options.cur_value  = strlower( user_defined.StatusBars.values or nUI_UnitLayout.StatusBars.values );
	self.HealthBar.Options.max_value  = strlower( user_defined.StatusBars.total or nUI_UnitLayout.StatusBars.total );
	self.HealthBar.Options.percent    = strlower( user_defined.StatusBars.percent or nUI_UnitLayout.StatusBars.percent );
	
	self.ManaBar.name                 = "nUI_Unit_"..self.unit_id.."_ManaBar";
	self.ManaBar.Options.show_name    = strlower( user_defined.StatusBars.name or nUI_UnitLayout.StatusBars.name );
	self.ManaBar.Options.cur_value    = strlower( user_defined.StatusBars.values or nUI_UnitLayout.StatusBars.values );
	self.ManaBar.Options.max_value    = strlower( user_defined.StatusBars.total or nUI_UnitLayout.StatusBars.total );
	self.ManaBar.Options.percent      = strlower( user_defined.StatusBars.percent or nUI_UnitLayout.StatusBars.percent );
	
	self.RageBar.name                 = "nUI_Unit_"..self.unit_id.."_RageBar";
	self.RageBar.Options.show_name    = strlower( user_defined.StatusBars.name or nUI_UnitLayout.StatusBars.name );
	self.RageBar.Options.cur_value    = strlower( user_defined.StatusBars.values or nUI_UnitLayout.StatusBars.values );
	self.RageBar.Options.max_value    = strlower( user_defined.StatusBars.total or nUI_UnitLayout.StatusBars.total );
	self.RageBar.Options.percent      = strlower( user_defined.StatusBars.percent or nUI_UnitLayout.StatusBars.percent );
	
	self.EnergyBar.name               = "nUI_Unit_"..self.unit_id.."_EnergyBar";
	self.EnergyBar.Options.show_name  = strlower( user_defined.StatusBars.name or nUI_UnitLayout.StatusBars.name );
	self.EnergyBar.Options.cur_value  = strlower( user_defined.StatusBars.values or nUI_UnitLayout.StatusBars.values );
	self.EnergyBar.Options.max_value  = strlower( user_defined.StatusBars.total or nUI_UnitLayout.StatusBars.total );
	self.EnergyBar.Options.percent    = strlower( user_defined.StatusBars.percent or nUI_UnitLayout.StatusBars.percent );
	
	self.FocusBar.name                = "nUI_Unit_"..self.unit_id.."_FocusBar";
	self.FocusBar.Options.show_name   = strlower( user_defined.StatusBars.name or nUI_UnitLayout.StatusBars.name );
	self.FocusBar.Options.cur_value   = strlower( user_defined.StatusBars.values or nUI_UnitLayout.StatusBars.values );
	self.FocusBar.Options.max_value   = strlower( user_defined.StatusBars.total or nUI_UnitLayout.StatusBars.total );
	self.FocusBar.Options.percent     = strlower( user_defined.StatusBars.percent or nUI_UnitLayout.StatusBars.percent );
	
	self.LatencyBar.name              = "nUI_Unit_"..self.unit_id.."_LatencyBar";
	self.LatencyBar.Options.show_name = "none";
	self.LatencyBar.Options.cur_value = "none";
	self.LatencyBar.Options.max_value = "none";
	self.LatencyBar.Options.percent   = "none";
	self.LatencyBar.Colors.bg         = { r=0, g=0, b=0, a=1 };
	
	self.CastBar.name                 = "nUI_Unit_"..self.unit_id.."_CastBar";
	self.CastBar.Options.show_name    = strlower( user_defined.CastingBars.name or nUI_UnitLayout.CastingBars.name );
	self.CastBar.Options.cur_value    = strlower( user_defined.CastingBars.values or nUI_UnitLayout.CastingBars.values );
	self.CastBar.Options.max_value    = strlower( user_defined.CastingBars.total or nUI_UnitLayout.CastingBars.total );
	self.CastBar.Options.percent      = strlower( user_defined.CastingBars.percent or nUI_UnitLayout.CastingBars.percent );
	self.CastBar.Colors.bg            = { r=0, g=0, b=0, a=1 };
	
	self.ChannelBar.name              = "nUI_Unit_"..self.unit_id.."_ChannelBar";
	self.ChannelBar.Options.show_name = strlower( user_defined.CastingBars.name or nUI_UnitLayout.CastingBars.name );
	self.ChannelBar.Options.cur_value = strlower( user_defined.CastingBars.values or nUI_UnitLayout.CastingBars.values );
	self.ChannelBar.Options.max_value = strlower( user_defined.CastingBars.total or nUI_UnitLayout.CastingBars.total );
	self.ChannelBar.Options.percent   = strlower( user_defined.CastingBars.percent or nUI_UnitLayout.CastingBars.percent );
	self.ChannelBar.Colors.bg         = { r=0, g=0, b=0, a=1 };
		
	if self.orientation ~= "right" then
		self.HealthBar.Options.grow  = "right";
		self.ManaBar.Options.grow    = "right";
		self.RageBar.Options.grow    = "right";
		self.EnergyBar.Options.grow  = "right";
		self.FocusBar.Options.grow   = "right";
		self.LatencyBar.Options.grow = "right";
		self.CastBar.Options.grow    = "right";
		self.ChannelBar.Options.grow = "right";
	else
		self.HealthBar.Options.grow  = "left";
		self.ManaBar.Options.grow    = "left";
		self.RageBar.Options.grow    = "left";
		self.EnergyBar.Options.grow  = "left";
		self.FocusBar.Options.grow   = "left";
		self.LatencyBar.Options.grow = "left";
		self.CastBar.Options.grow    = "left";
		self.ChannelBar.Options.grow = "left";
	end				
end
